==> gm-1.6.3_Linux.tar.gz: gm-1.6.3_Linux/drivers/linux/gm/gm_arch_linux.h <==
#ifndef _gm_arch_linux_h
#define _gm_arch_linux_h


#ifndef LINUX_VERSION_CODE
#include <linux/version.h>
#endif

#ifndef VERSION_CODE
#define VERSION_CODE(vers,rel,seq) ( ((vers)<<16) | ((rel)<<8) | (seq) )
#endif

#if LINUX_VERSION_CODE < VERSION_CODE(2,2,0)
#warning "This driver does not support Linux 2.1.x or earlier."
#error "Please use Linux 2.4.x or 2.2.x."
#elif LINUX_VERSION_CODE < VERSION_CODE(2,3,0)
#define LINUX_22 1
#define LINUX_24 0
#define LINUX_XX 22
#elif LINUX_VERSION_CODE < VERSION_CODE(2,4,0)
#warning "The use of unstable kernel release is discouraged."
#warning "Myricom help line will not support such configuration."
#define LINUX_22 0
#define LINUX_24 1
#define LINUX_XX 24
#elif LINUX_VERSION_CODE < VERSION_CODE(2,5,0)
#define LINUX_22 0
#define LINUX_24 1
#define LINUX_XX 24
#elif LINUX_VERSION_CODE < VERSION_CODE(2,7,0)
#define LINUX_22 0
#define LINUX_24 1
#define LINUX_XX 26
#else
#error "Unsupported Linux kernel release."
#endif


#define REGISTER_SYMTAB(tab)
#include <linux/poll.h>
#include <asm/uaccess.h>
#include <asm/io.h>
#include <linux/types.h>

#ifdef __alpha__
#include <asm/machvec.h>
#if LINUX_22
#include <asm/core_pyxis.h>
#endif
#endif

#if LINUX_VERSION_CODE >= VERSION_CODE(2,2,18)
#include <linux/spinlock.h>
#endif

#if LINUX_VERSION_CODE < VERSION_CODE(2,2,18)
typedef struct wait_queue *wait_queue_head_t;
#define init_waitqueue_head(a) init_waitqueue(a)
#define init_MUTEX(s) do { *(s) = MUTEX; } while (0)
#define DECLARE_WAITQUEUE(a,b) struct wait_queue a = {(b), 0}
#endif


#if LINUX_XX <= 22

/* definition implementing the 2.4 and later APIS on top of 2.2 APIS */
#include <linux/pci.h>

#define PCI_ANY_ID (~0)
#define VXX_MAX_DEV_PER_DRIVER 16

struct pci_device_id
{
  unsigned int vendor;		/* Vendor ID or PCI_ANY_ID */
  unsigned int device;		/* Device ID or PCI_ANY_ID */
  unsigned int subvendor;	/* Subsystem ID's or PCI_ANY_ID */
  unsigned int subdevice;	/* Subsystem ID's or PCI_ANY_ID */
  unsigned int class;		/* (class,subclass,prog-if) triplet */
  unsigned int class_mask;
  unsigned long driver_data;	/* Data private to the driver */
};

struct pci_driver
{
  char *name;
  const struct pci_device_id *id_table;
  int (*probe) (struct pci_dev * dev, const struct pci_device_id * id);
  void (*remove) (struct pci_dev * dev);
  struct pci_dev *devs[VXX_MAX_DEV_PER_DRIVER];
};


int pci_module_init (struct pci_driver *);
void pci_unregister_driver (struct pci_driver *);

#define module_init(x) int init_module(void) { return x(); }
#define module_exit(x) void cleanup_module(void) { x(); }
#define dev_kfree_skb_any(a) dev_kfree_skb(a)
#define dev_kfree_skb_irq(a) dev_kfree_skb(a)
#define pci_dev_driver_data(d) ((void*)(d)->base_address[5])
#define vma_private_data(v) ((void*)(v)->vm_pte)
#define gm_linux_mem_map_reserve(a) set_bit(PG_reserved, &(a)->flags)
#define gm_linux_mem_map_unreserve(a) clear_bit(PG_reserved, &(a)->flags)
#define virt_to_page(a) (mem_map+MAP_NR(a))
#define dev_start(a,b) do { (a)->start = (b) ; } while (0)
#define vma_get_pgoff(vma) ((vma)->vm_offset / PAGE_SIZE)
#define netif_wake_queue(dev) \
do { (dev)->tbusy = 0; mark_bh(NET_BH); } while (0)
#define netif_start_queue(dev) do { (dev)->tbusy = 0; } while (0)
#define netif_stop_queue(dev) do { (dev)->tbusy = 1; } while (0)
#define netif_queue_stopped(dev) ((dev)->tbusy)

#define net_device_stats enet_statistics
#define net_device device

#define pci_resource_start(pdev,i) \
((pdev)->base_address[i] & PCI_BASE_ADDRESS_MEM_MASK)

struct net_device *gm_linux_dev_alloc (char *, int *, int num);

#define gm_linux_mm_lock(a)	/*nothing */
#define gm_linux_mm_unlock(a)	/*nothing */
#define gm_linux_free_page(a) free_page(page_address(a))
#define pci_set_dma_mask(pdev,a) (0)

#define devfs_register(dir,name,flags,major,minor,mode,ops,data) 0
#define devfs_unregister(handle)
#define devfs_register_chrdev register_chrdev
#define devfs_unregister_chrdev unregister_chrdev
typedef void * devfs_handle_t;

#define THIS_MODULE (&__this_module)

#define gm_linux_flush_scheduled_tasks() schedule()

extern inline 
int schedule_task(struct tq_struct *task)
{
  queue_task(task, &tq_scheduler);
  return 1;
}

#define gm_mmap_up_write(a)
#define gm_mmap_down_write(a)

#endif

#if LINUX_XX >= 24

/* The stuff here is mainly to support early 2.4 kernels while using
   some functionality introduced only later in the series. */

#include <linux/devfs_fs_kernel.h>
#include <asm/system.h>
#include <linux/module.h>
#include <linux/kdev_t.h>

#if LINUX_VERSION_CODE < VERSION_CODE(2,4,3)
#define pci_set_dma_mask(pdev,a) (0)
#endif

#define pci_dev_driver_data(d) ((d)->driver_data)
#define vma_get_pgoff(vma) ((vma)->vm_pgoff)
#define dev_start(a,b)
#define gm_linux_mem_map_reserve(a) mem_map_reserve(a)
#define gm_linux_mem_map_unreserve(a) mem_map_unreserve(a)
#define gm_linux_dev_alloc(s,err,num) dev_alloc(s,err)


#if defined(HAVE_MMAP_UP_WRITE)
#define gm_mmap_up_write mmap_up_write
#define gm_mmap_down_write mmap_down_write
#elif LINUX_VERSION_CODE < VERSION_CODE(2,4,3)
#define gm_mmap_up_write(a) up(&(a)->mmap_sem)
#define gm_mmap_down_write(a) down(&(a)->mmap-sem)
#elif defined __alpha__ && LINUX_VERSION_CODE < VERSION_CODE(2,4,7)
/* __down_write and __up_write not exported */
/* not getting the semaphore is still safe because we have the page_table_lock
   and the kernel_lock anyway */
#define gm_mmap_up_write(a)
#define gm_mmap_down_write(a)
#elif defined __powerpc__ && LINUX_VERSION_CODE == VERSION_CODE(2,4,4)
/* __down_write and __up_write not defined/exported */
/* not getting the semaphore is still safe for this version
   because we have the page_table_lock and the kernel_lock anyway */
#define gm_mmap_up_write(a)
#define gm_mmap_down_write(a)
#else
#define gm_mmap_up_write(a) up_write(&(a)->mmap_sem)
#define gm_mmap_down_write(a) down_write(&(a)->mmap_sem)
#endif
#define gm_linux_mm_lock(a) do { gm_mmap_down_write(a); \
   spin_lock(&(a)->page_table_lock); } while (0)
#define gm_linux_mm_unlock(a) do { spin_unlock(&(a)->page_table_lock); \
  gm_mmap_up_write(a); } while (0)
#define gm_linux_free_page(a) __free_page(a)

#ifndef read_cr4
#define read_cr4() ({				\
        unsigned int __dummy;			\
        __asm__(				\
                "movl %%cr4,%0\n\t"		\
                :"=r" (__dummy));		\
        __dummy;				\
})
#endif

#define gm_linux_flush_scheduled_tasks() flush_scheduled_tasks()

#endif

/* version independant definitions */

#ifndef MODULE_LICENSE
#define MODULE_LICENSE(license)
#endif

#ifndef HAVE_PTE_OFFSET_MAP
#define pte_offset_map pte_offset
#define pte_offset_map_nested pte_offset
#define pte_unmap(pte)
#define pte_unmap_nested(pte)
#elif !defined HAVE_PTE_OFFSET_MAP_NESTED
#define pte_offset_map_nested pte_offset_map2
#define pte_unmap_nested pte_unmap2
#endif

#ifdef HAVE_REMAP_PAGE_RANGE_5ARGS
#define gm_linux_remap_page_range(vma,start,phys,len,prot) \
   remap_page_range ((vma),(start),(phys),(len),(prot))
#else
#define gm_linux_remap_page_range(vma,start,phys,len,prot) \
   remap_page_range ((start),(phys),(len),(prot))
#endif

#ifndef minor
#define minor(a) MINOR(a)
#endif

#endif

==> gm-2.0.8_Linux.tar.gz: gm-2.0.8_Linux/drivers/linux/gm/gm_arch_linux.h <==
#ifndef _gm_arch_linux_h
#define _gm_arch_linux_h


#ifndef LINUX_VERSION_CODE
#include <linux/version.h>
#endif

#ifndef VERSION_CODE
#define VERSION_CODE(vers,rel,seq) ( ((vers)<<16) | ((rel)<<8) | (seq) )
#endif

#if LINUX_VERSION_CODE < VERSION_CODE(2,2,0)
#warning "This driver does not support Linux 2.1.x or earlier."
#error "Please use Linux 2.4.x or 2.2.x."
#elif LINUX_VERSION_CODE < VERSION_CODE(2,3,0)
#define LINUX_22 1
#define LINUX_24 0
#define LINUX_XX 22
#elif LINUX_VERSION_CODE < VERSION_CODE(2,4,0)
#warning "The use of unstable kernel release is discouraged."
#warning "Myricom help line will not support such configuration."
#define LINUX_22 0
#define LINUX_24 1
#define LINUX_XX 24
#elif LINUX_VERSION_CODE < VERSION_CODE(2,5,0)
#define LINUX_22 0
#define LINUX_24 1
#define LINUX_XX 24
#elif LINUX_VERSION_CODE < VERSION_CODE(2,7,0)
#define LINUX_22 0
#define LINUX_24 1
#define LINUX_XX 26
#else
#error "Unsupported Linux kernel release."
#endif


#define REGISTER_SYMTAB(tab)
#include <linux/poll.h>
#include <linux/sched.h>
#include <asm/uaccess.h>
#include <asm/io.h>
#include <linux/types.h>

#ifdef __alpha__
#include <asm/machvec.h>
#if LINUX_22
#include <asm/core_pyxis.h>
#endif
#endif

#if LINUX_VERSION_CODE >= VERSION_CODE(2,2,18)
#include <linux/spinlock.h>
#endif

#if LINUX_VERSION_CODE < VERSION_CODE(2,2,18)
typedef struct wait_queue *wait_queue_head_t;
#define init_waitqueue_head(a) init_waitqueue(a)
#define init_MUTEX(s) do { *(s) = MUTEX; } while (0)
#define DECLARE_WAITQUEUE(a,b) struct wait_queue a = {(b), 0}
#endif


#if LINUX_XX <= 22

/* definition implementing the 2.4 and later APIS on top of 2.2 APIS */
#include <linux/pci.h>

#define PCI_ANY_ID (~0)
#define VXX_MAX_DEV_PER_DRIVER 16

struct pci_device_id
{
  unsigned int vendor;		/* Vendor ID or PCI_ANY_ID */
  unsigned int device;		/* Device ID or PCI_ANY_ID */
  unsigned int subvendor;	/* Subsystem ID's or PCI_ANY_ID */
  unsigned int subdevice;	/* Subsystem ID's or PCI_ANY_ID */
  unsigned int class;		/* (class,subclass,prog-if) triplet */
  unsigned int class_mask;
  unsigned long driver_data;	/* Data private to the driver */
};

struct pci_driver
{
  char *name;
  const struct pci_device_id *id_table;
  int (*probe) (struct pci_dev * dev, const struct pci_device_id * id);
  void (*remove) (struct pci_dev * dev);
  struct pci_dev *devs[VXX_MAX_DEV_PER_DRIVER];
};


int pci_module_init (struct pci_driver *);
void pci_unregister_driver (struct pci_driver *);

#define module_init(x) int init_module(void) { return x(); }
#define module_exit(x) void cleanup_module(void) { x(); }
#define dev_kfree_skb_any(a) dev_kfree_skb(a)
#define dev_kfree_skb_irq(a) dev_kfree_skb(a)
#define pci_dev_driver_data(d) ((void*)(d)->base_address[5])
#define vma_private_data(v) ((void*)(v)->vm_pte)
#define gm_linux_mem_map_reserve(a) set_bit(PG_reserved, &(a)->flags)
#define gm_linux_mem_map_unreserve(a) clear_bit(PG_reserved, &(a)->flags)
#define virt_to_page(a) (mem_map+MAP_NR(a))
#define dev_start(a,b) do { (a)->start = (b) ; } while (0)
#define vma_get_pgoff(vma) ((vma)->vm_offset / PAGE_SIZE)
#define netif_wake_queue(dev) \
do { (dev)->tbusy = 0; mark_bh(NET_BH); } while (0)
#define netif_start_queue(dev) do { (dev)->tbusy = 0; } while (0)
#define netif_stop_queue(dev) do { (dev)->tbusy = 1; } while (0)
#define netif_queue_stopped(dev) ((dev)->tbusy)

#define net_device_stats enet_statistics
#define net_device device

#define pci_resource_start(pdev,i) \
((pdev)->base_address[i] & PCI_BASE_ADDRESS_MEM_MASK)

struct net_device *gm_linux_dev_alloc (char *, int *, int num);

#define gm_linux_mm_lock(a)	/*nothing */
#define gm_linux_mm_unlock(a)	/*nothing */
#define gm_linux_free_page(a) free_page(page_address(a))
#define pci_set_dma_mask(pdev,a) (0)

#define devfs_register(dir,name,flags,major,minor,mode,ops,data) 0
#define devfs_unregister(handle)
#define devfs_register_chrdev register_chrdev
#define devfs_unregister_chrdev unregister_chrdev
typedef void * devfs_handle_t;

#define THIS_MODULE (&__this_module)

#define gm_linux_flush_scheduled_tasks() schedule()

extern inline 
int schedule_task(struct tq_struct *task)
{
  queue_task(task, &tq_scheduler);
  return 1;
}

#define gm_mmap_up_write(a)
#define gm_mmap_down_write(a)

#define gm_linux_page_inode(page) ((page)->inode)

#endif

#if LINUX_XX >= 24

/* The stuff here is mainly to support early 2.4 kernels while using
   some functionality introduced only later in the series. */

#include <linux/devfs_fs_kernel.h>
#include <asm/system.h>
#include <linux/module.h>
#include <linux/kdev_t.h>

#if LINUX_VERSION_CODE < VERSION_CODE(2,4,3)
#define pci_set_dma_mask(pdev,a) (0)
#endif

#define pci_dev_driver_data(d) ((d)->driver_data)
#define vma_get_pgoff(vma) ((vma)->vm_pgoff)
#define dev_start(a,b)
#define gm_linux_mem_map_reserve(a) mem_map_reserve(a)
#define gm_linux_mem_map_unreserve(a) mem_map_unreserve(a)
#define gm_linux_dev_alloc(s,err,num) dev_alloc(s,err)


#if defined(HAVE_MMAP_UP_WRITE)
#define gm_mmap_up_write mmap_up_write
#define gm_mmap_down_write mmap_down_write
#elif LINUX_VERSION_CODE < VERSION_CODE(2,4,3)
#define gm_mmap_up_write(a) up(&(a)->mmap_sem)
#define gm_mmap_down_write(a) down(&(a)->mmap_sem)
#elif defined __alpha__ && LINUX_VERSION_CODE < VERSION_CODE(2,4,7)
/* __down_write and __up_write not exported */
/* not getting the semaphore is still safe because we have the page_table_lock
   and the kernel_lock anyway */
#define gm_mmap_up_write(a)
#define gm_mmap_down_write(a)
#elif defined __powerpc__ && LINUX_VERSION_CODE == VERSION_CODE(2,4,4)
/* __down_write and __up_write not defined/exported */
/* not getting the semaphore is still safe for this version
   because we have the page_table_lock and the kernel_lock anyway */
#define gm_mmap_up_write(a)
#define gm_mmap_down_write(a)
#else
#define gm_mmap_up_write(a) up_write(&(a)->mmap_sem)
#define gm_mmap_down_write(a) down_write(&(a)->mmap_sem)
#endif
#define gm_linux_mm_lock(a) do { gm_mmap_down_write(a); \
   spin_lock(&(a)->page_table_lock); } while (0)
#define gm_linux_mm_unlock(a) do { spin_unlock(&(a)->page_table_lock); \
  gm_mmap_up_write(a); } while (0)
#define gm_linux_free_page(a) __free_page(a)

#ifndef read_cr4
#define read_cr4() ({				\
        unsigned int __dummy;			\
        __asm__(				\
                "movl %%cr4,%0\n\t"		\
                :"=r" (__dummy));		\
        __dummy;				\
})
#endif

#define gm_linux_flush_scheduled_tasks() flush_scheduled_tasks()
#define gm_linux_page_inode(page) ((page)->mapping ? (page)->mapping->host : 0)
#endif

/* version independant definitions */

#ifndef MODULE_LICENSE
#define MODULE_LICENSE(license)
#endif

/*
 * Figure out what, if any, high-memory support routines to use
 */
#ifdef HAVE_PTE_KUNMAP /* suse version */

#define pte_unmap pte_kunmap
#define pte_unmap_nested pte_unmap
#define pte_offset_map pte_offset
#define pte_offset_map_nested pte_offset

#elif defined HAVE_PTE_OFFSET_MAP_NESTED /* linux 2.5 version */

/* nothing to do since we use this interface in our code */

#elif defined HAVE_PTE_OFFSET_MAP /* redhat version */

#define pte_offset_map_nested pte_offset_map2
#define pte_unmap_nested pte_unmap2

#else /* not using high pte stuff */

#define pte_offset_map pte_offset
#define pte_offset_map_nested pte_offset
#define pte_unmap(pte)
#define pte_unmap_nested(pte)

#endif

#ifdef HAVE_REMAP_PAGE_RANGE_5ARGS
#define gm_linux_remap_page_range(vma,start,phys,len,prot) \
   remap_page_range ((vma),(start),(phys),(len),(prot))
#else
#define gm_linux_remap_page_range(vma,start,phys,len,prot) \
   remap_page_range ((start),(phys),(len),(prot))
#endif

#ifndef minor
#define minor(a) MINOR(a)
#endif

#ifndef set_current_state
#define set_current_state(x) do { current->state = (x); } while (0)
#endif

#endif
